/**
 * domo dev server
 */

const fs = require('fs-extra');
const path = require('path');
const _ = require('lodash');
const Domo = require('ryuu-client');
const request = require('request');
const koa = require('koa');
const Router = require('koa-router');
const body_parser = require('koa-bodyparser');
const api = new Router();
const session = require('./util/session');
const sidUtil = require('./util/sid');
const appStructure = require('./util/appStructure');
const getRawBody = require('raw-body');
const helpers = require('./util/helpers');
const mostRecent = session.getMostRecentLogin;

const app = module.exports = koa();

api.get('/', checkContext);
api.all('/data/*', proxyRequest);
api.all('/domo/*', setHeaders);
app.use(body_parser({
  detectJSON: function(ctx) {
    return false;
  }
}));
app.use(api.middleware());

function* checkContext() {
  const manifest = fs.readJsonSync(path.resolve(process.cwd() + '/' + (app.options.manifest || 'manifest.json')));
  const hasThumbnail = appStructure.hasThumbnail();
  let isValidSession = yield session.check(app.domo);
  let isValidContext = true;
  let context = yield helpers.createContext(manifest.id, manifest.mapping, app.domo);
  let userId = app.options.userId || 1;
  let customerId = 1;
  let template = _.template(fs.readFileSync(path.resolve(__dirname + '/../server/domodev/index.html')));
  if (!context) {
    isValidContext = false;
    context = { id: 0 };
  }

  if (app.domo.sid) {
    const sid = sidUtil.parse(app.domo.sid);
    userId = app.options.userId || sid.userId;
    customerId = customerId || sid.customerId;
  }
  const sizeMultiplier = {
    width: 235,
    height: 290
  }
  // sizes consistent with Domo Web
  const width = 235 * manifest.size.width - 10 + 'px';
  const height = 290 * manifest.size.height - 40 + 'px';
  const fullpage = manifest.fullpage;

  var url = `index.html?userId=${userId}&customer=${customerId}&locale=en-US&platform=desktop&context=${context.id}`;
  this.body = template({
    manifest,
    width,
    height,
    url,
    isValidSession,
    isValidContext,
    hasThumbnail,
    fullpage
  });
}

// Proxy requests to the correct domoapps service for the environment they're logged into.

function* proxyRequest() {
  const manifest = fs.readJsonSync(path.resolve(process.cwd() + '/' + (app.options.manifest || 'manifest.json')));
  const baseUrl = yield helpers.getDomoappsDomain(app.domo);
  const j = request.jar();
  const method = this.request.method;
  let url = baseUrl + this.request.url;

  const headers = _.merge({
    referer: this.request.headers.referer,
    accept: this.request.headers.accept,
    'Content-Type': this.request.headers['content-type'] || this.request.headers['Content-Type'] || 'application/json'
  }, app.domo.getAuthHeader());


  let body = JSON.stringify(this.request.body);
  let alias;
  if (url.indexOf("dataversions") !== -1) {
    body = yield getRawBody(this.req, {
      length: this.length,
      limit: '1mb',
      encoding: this.charset
    });
    url = url.replace("/dataversions", "/dataversions/domodev");
    url += "&designId=" + manifest.id;
    alias = url.match(/data\/(v1|v2|v3)\/(.*)\/dataversions/)[2];
    url += "&datasetId=" + _.find(manifest.mapping, { alias: alias }).dataSetId;
  }

  this.body = request({
    url: url,
    method: method,
    jar: j,
    body: body,
    headers: headers,
  });
}

function* setHeaders() {
  const baseUrl = yield helpers.getDomoappsDomain(app.domo);
  const j = request.jar();
  const url = baseUrl + this.request.url;

  let headers = _.merge({
    referer: this.request.headers.referer,
    accept: this.request.headers.accept,
    'Content-Type': this.request.headers['content-type'] || this.request.headers['Content-Type'] || 'application/json'
  }, app.domo.getAuthHeader());

  this.body = request({ url: url, jar: j, headers: headers });
}

// app.use(koaWebpack());
// app.use(mount('/domodev', serve(path.resolve(__dirname + '/../server/domodev'))));
// app.use(mount(serve('.')));