/**
 * Server utils/helpers
 */
const request = require('request');
const Domo = require('ryuu-client');
const proxyaddr = require('proxy-addr');

module.exports = {

  getCustomer: function(app) {
    const regexp = /([\w]+)[\.|-]/;
    return app.instance.match(regexp)[1];
  },

  getEnv: function(app) {
    const regexp = /([-_\w]+)\.(.*)/;
    return app.instance.match(regexp)[2];
  },

  getMiddleware: function() {
    return [function(req, res, next) {
      if (proxyaddr(req, '127.0.0.1') !== '::1') {
        Error.stackTraceLimit = 0;
        next(new Error('Access Denied'));
      } else {
        next();
      }
    }];
  },

  createContext: function(designId, mapping, app) {
    return new Promise(function(resolve, reject) {
      const options = {
        url: 'https://' + app.instance + '/domoapps/apps/v2/contexts',
        method: 'POST',
        json: { designId: designId, mapping: mapping },
        headers: app.getAuthHeader()
      };

      request(options, function(err, res) {
        resolve(res.body[0]);
      });

    });
  },

  /**
   * Fetch domoapps domain from maestro.
   * Fallback to inferring it by the instance URL.
   */
  getDomoappsDomain: function(app) {
    const uuid = Domo.createUUID();
    const j = request.jar();
    const auth = 'SID="' + app.sid + '"';
    const cookie = request.cookie(auth);
    j.setCookie(cookie, 'https://' + app.instance);

    const options = {
      url: 'https://' + app.instance + '/api/content/v1/mobile/environment',
      headers: app.getAuthHeader()
    }

    return new Promise(function(resolve, reject) {
      request(options, function(err, res) {
        if (res.statusCode === 200) {
          const url = 'https://' + uuid + '.' + JSON.parse(res.body).domoappsDomain;
          resolve(url);
        } else {
          resolve('https://' + uuid + '.domoapps.' + getEnv());
        }
      });
    });
  }
};
